/***************************************************************   
IMSE1-P.PRG

written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
www.ssc.wisc.edu/~bhansen
bhansen@ssc.wisc.edu

This Gauss program plots MISE by bandwidth and kernel order
as described in the  paper
"Exact Mean Integrated Squared Error of Higher-Order Kernel Estimators"
and creates Figures 2-4 for that paper.

The program uses the Gauss file "imse1.gcg" which is created by first
running the program "imse1-c.prg".

*****************************************************************/

use imse1;

let ns= 50 200 500 1000;
nn=rows(ns);

mm=10;
_print = 0;		@ set to 1 to send all graphs to the printer @

/*****************************************************************/

imse=arrayalloc(_mmax|nn|(sn+1)|rn|hn,0);
imsed=arrayalloc(_mmax|nn|hn,0);
imset=arrayalloc(_mmax|nn|hn,0);
mse=arrayalloc(_mmax|nn|(sn+1)|rn,0);
msed=zeros(_mmax,nn);
mset=zeros(_mmax,nn);

for _m (1,mm,1);

{p,mu,sig} = models(_m);
sd=mn_sd(_m);
s0=sqrt((sig.^2)+(sig.^2)');
x0=(mu-mu')./s0;
f0=expt(-(x0.^2)/2)/sqrt(2*pi)./s0;
i0=p'f0*p;


h0=TrapRuleThumb(50,sd);
h1=hs*h0;
i1=intd1[_m,.]';
i2=i1;
for ni (1,nn,1);
  n=ns[ni];
  im=(1/pi)./h1+i0*n+i2*(n-1)-i1*2*n;
  imsed[_m,ni,1:hn]=im';
  msed[_m,ni]=minc(im);
endfor;

i1=intt1[_m,.]';
i2=intt2[_m,.]';
for ni (1,nn,1);
  n=ns[ni];
  im=(4/3/pi)./h1+i0*n+i2*(n-1)-i1*2*n;
  imset[_m,ni,1:hn]=im';
  mset[_m,ni]=minc(im);
endfor;

for ri (1,rn,1);
  r=rs[ri];
  h0=GaussRuleThumb(50,r,sd);
  h1=hs*h0;
  i1=arraytomat(int1[_m,1,ri,.])';
  i2=arraytomat(int2[_m,1,ri,.])';
  for ni (1,nn,1);
    n=ns[ni];
    im=GaussRoughness(r)./h1+i0*n+i2*(n-1)-i1*2*n;
    imse[_m,ni,1,ri,1:hn]=im';
    mse[_m,ni,1,ri]=minc(im);
  endfor;
endfor;

for ri (1,rn,1);
  r=rs[ri];
  for si (1,sn,1);
    s=ss[si];
    h0=PolyRuleThumb(50,r,s,sd);
    h1=hs*h0;
    i1=arraytomat(int1[_m,si+1,ri,.])';
    i2=arraytomat(int2[_m,si+1,ri,.])';
    for ni (1,nn,1);
      n=ns[ni];
      im=PolyRoughness(r,s)./h1+i0*n+i2*(n-1)-i1*2*n;
      imse[_m,ni,si+1,ri,1:hn]=im';
      mse[_m,ni,si+1,ri]=minc(im);
    endfor;
  endfor;
endfor;

endfor;


@ Graph of MISE(h) @
for _m (1,mm,1);
graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_paxes=1;
_protate=1;
_plwidth=3;
_paxht=.2;
_pnumht=.15;
ylabel("MISE");
begwind;
window(nn,sn+2,0);
makewind(6.855,9,0,0,1);
setwind(1);
for ni (1,nn,1);
  yy = "" $+ "G]2r[  " $+ "n = " $+ ftocv(ns[ni],1,0);
  xlabel(yy);
  im=ln(1+arraytomat(imse[_m,ni,1,.,.])');
  xy(hs,im);nextwind;
  for si (1,sn,1);
    yy = "" $+ "K]2r," $+ ftocv(ss[si],1,0) $+ "[  " $+ "n = " $+ ftocv(ns[ni],1,0);
    xlabel(yy);
    im=ln(1+arraytomat(imse[_m,ni,1+si,.,.])');
    xy(hs,im);nextwind;
  endfor;
  yy = "" $+ "D, \202l\201]2[  " $+ "n = " $+ ftocv(ns[ni],1,0);
  xlabel(yy);
  im=ln(1+arraytomat(imsed[_m,ni,.])')~ln(1+arraytomat(imset[_m,ni,.])');
  xy(hs,im);nextwind;
endfor;
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
3.0 8.8 .2 0 2 15 0 ;
_pmsgstr = "" $+ "MISE(h), Density #" $+ ftocv(_m,1,0);
draw;
if _print==1; graphprt("-p"); endif;
endwind;
endfor;



@ Graphs of Exact MISE with Oracle Bandwidths as function of kernel order @
for _m (1,mm,1);
graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_protate=1;
xtics(2*rs[1],2*rs[rn],2,1);
let _pltype= 6 3 1 5 2 4;
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_pypmax=1;
_plegstr= "" $+ "G]2r[\000";
for si (1,sn,1);
  _plegstr= _plegstr $+ "K]2r," $+ ftocv(ss[si],1,0) $+ "[\000";
endfor;
_plegstr= _plegstr $+ "D\000\202l\201]2[";
ylabel("Relative MISE");
xlabel("2r");
_pmsgctl=0;
_paxes=1;
begwind;
window(2,2,1);
makewind(6.855,9,0,0,1);
setwind(1);
ni=1;do while ni<=nn;
 if ni==2; let _plegctl= 2 5 5 6.3;
 else; _plegctl=0;
 endif;
 tj="n=" $+ ftocv(ns[ni],1,0);
 title(tj);
 ar=arraytomat(mse[_m,ni,.,.])';
 g=minc(vec(ar));
 ar=(ar~(ones(rn,1)*(msed[_m,ni]~mset[_m,ni])))./g;
 ytics(1,1.3,.1,10);
 xy(rs*2,ar);
 nextwind;
ni=ni+1;endo;
setwind(nn+1);
_plegctl=0;
title("");
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
3 8.7 .2 0 2 15 0 ;
_pmsgstr = "" $+ "Density #" $+ ftocv(_m,1,0);
draw;
if _print==1; graphprt("-p"); endif;
endwind;
endfor;


@ Figures for Paper @
@ Density # 1 @
_m=1;
graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_protate=1;
xtics(2*rs[1],2*rs[rn],2,1);
let _pltype= 6 3 1 5 2 4;
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_pypmax=1;
_plegstr= "" $+ "G]2r[\000";
for si (1,sn,1);
  _plegstr= _plegstr $+ "K]2r," $+ ftocv(ss[si],1,0) $+ "[\000";
endfor;
_plegstr= _plegstr $+ "D\000\202l\201]2[";
ylabel("Relative MISE");
xlabel("2r");
let _pmsgctl[1,7] = 3 7 .2 0 2 15 0 ;
_paxes=1;
begwind;
window(2,2,1);
makewind(6.855,9,0,0,1);
setwind(1);
ni=1;do while ni<=nn;
 if ni==2; let _plegctl= 2 5 5 5.8;
 else; _plegctl=0;
 endif;
 _pmsgstr="n=" $+ ftocv(ns[ni],1,0);
 ar=arraytomat(mse[_m,ni,.,.])';
 g=minc(vec(ar));
 ar=(ar~(ones(rn,1)*(msed[_m,ni]~mset[_m,ni])))./g;
 ytics(1,1.3,.1,10);
 xy(rs*2,ar);
 nextwind;
ni=ni+1;endo;
setwind(nn+1);
_plegctl=0;
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
2.75 8.7 .2 0 2 15 0 ;
_pmsgstr = "" $+ "Figure 2: MISE, Density #" $+ ftocv(_m,1,0);
draw;
if _print==1; graphprt("-p"); endif;
endwind;

@ Density # 3 @
_m=3;
graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_protate=1;
xtics(2*rs[1],2*rs[rn],2,1);
let _pltype= 6 3 1 5 2 4;
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_pypmax=1;
_plegstr= "" $+ "G]2r[\000";
for si (1,sn,1);
  _plegstr= _plegstr $+ "K]2r," $+ ftocv(ss[si],1,0) $+ "[\000";
endfor;
_plegstr= _plegstr $+ "D\000\202l\201]2[";
ylabel("Relative MISE");
xlabel("2r");
_paxes=1;
let _pmsgctl[1,7] = 3 7 .2 0 2 15 0 ;
begwind;
window(2,2,1);
makewind(6.855,9,0,0,1);
setwind(1);
ni=1;do while ni<=nn;
 if ni==2; let _plegctl= 2 5 5 5.8;
 else; _plegctl=0;
 endif;
 _pmsgstr="n=" $+ ftocv(ns[ni],1,0);
 ar=arraytomat(mse[_m,ni,.,.])';
 g=minc(vec(ar));
 ar=(ar~(ones(rn,1)*(msed[_m,ni]~mset[_m,ni])))./g;
 ytics(1,1.3,.1,10);
 xy(rs*2,ar);
 nextwind;
ni=ni+1;endo;
setwind(nn+1);
_plegctl=0;
title("");
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
2.75 8.7 .2 0 2 15 0 ;
_pmsgstr = "" $+ "Figure 3: MISE, Density #" $+ ftocv(_m,1,0);
draw;
if _print==1; graphprt("-p"); endif;
endwind;


@ Graphs of Regret @
graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_protate=1;
_pltype=6;
xtics(2*rs[1],2*rs[rn],2,1);
_plwidth=zeros(mm,1)|8;
_paxht=.2;
_pnumht=.15;
_pypmax=1;
ytics(0,.3,.1,10);

@ Gaussian @
xlabel("2r");
ylabel("Regret");
_paxes=1;
let _pmsgctl[1,7] = 3 7 .2 0 2 15 0 ;
begwind;
window(2,2,1);
makewind(6.855,9,0,0,1);
setwind(1);
ni=1;do while ni<=nn;
 _pmsgstr="n=" $+ ftocv(ns[ni],1,0);
 aa=zeros(rn,mm);
 for m (1,mm,1);
  ar=arraytomat(mse[m,ni,.,.])';
  aa[.,m]=ar[.,1]./minc(vec(ar));
 endfor;
 aa=ln(aa);
 am=maxc(aa');
 xy(rs*2,aa~am);
 nextwind;
ni=ni+1;endo;
setwind(nn+1);
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
1.25 8.7 .2 0 2 15 0 ;
_pmsgstr = "" $+ "Figure 4: Regret Using Gaussian Kernels";
draw;
if _print==1; graphprt("-p"); endif;
endwind;


@ Polynomial @
xlabel("2r");
ylabel("Regret");
_paxes=1;
let _pmsgctl[1,7] = 3 7 .2 0 2 15 0 ;
begwind;
window(2,2,1);
makewind(6.855,9,0,0,1);
setwind(1);
ni=1;do while ni<=nn;
 _pmsgstr="n=" $+ ftocv(ns[ni],1,0);
 aa=zeros(rn,mm);
 for m (1,mm,1);
  ar=arraytomat(mse[m,ni,.,.])';
  aa[.,m]=ar[.,1+sn]./minc(vec(ar));
 endfor;
 aa=ln(aa);
 am=maxc(aa');
 xy(rs*2,aa~am);
 nextwind;
ni=ni+1;endo;
setwind(nn+1);
ylabel("");
xlabel("");
_paxes = 0;
let _pmsgctl[1,7] = 
2 8.7 .2 0 2 15 0 ;
_pmsgstr = "" $+ "Regret Using Polynomial Kernel";
draw;
if _print==1; graphprt("-p"); endif;
endwind;


/*****************************************************/
