/***************************************************************   
IMSE2-C.PRG

written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
www.ssc.wisc.edu/~bhansen
bhansen@ssc.wisc.edu

This Gauss program numerically and analytically calculates integrals 
for evaluation of MISE as described in the  paper
"Exact Mean Integrated Squared Error of Higher-Order Kernel Estimators"

The program loads the file "legendre.fmt" which contains points of evaluation
for Gauss-Legendre numerical integration.

The program creates the output "imse2.gcg" which is needed for the program 
"imse2-p.prg" 

This program takes several hours to run.

*****************************************************************/

let rs = 1 2 3 4 5 6;
rn=rows(rs);
s = 2 ;

loadm legendre;
q=4;
xx=packr(legendre[.,2*q-1]);
ww=packr(legendre[.,2*q]);
tol=.000001;

_mmax=6;

ms=miss(0,0);

/*****************************************************************/

let bbs=10 10 40 30 30 10 10 10 10 30 110 50 80 90 50;

@ Gaussian Kernels @

hn=300;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

intg1=arrayinit(_mmax|rn|hn,ms);
intg2=arrayinit(_mmax|rn|hn,ms);
intg1a=arrayinit(_mmax|rn|hn,ms);
intg2a=arrayinit(_mmax|rn|hn,ms);

for _m (1,_mmax,1);

bb=bbs[_m];
x=((-rev(xx))|xx)*(bb/2)+bb/2;
w=(rev(ww)|ww)*(bb/2);

{p,mu,sig} = models(_m);
q=rows(p);
fs=abs((expt(-((x*(sig')).^2)/2).*expt(x*sqrt(-1)*(mu')))*p).^2;
sd=mn_sd(_m);
sig2=(sig.^2)+(sig.^2)';
s0=sqrt(sig2);
x0=(mu-mu')./s0;
f0=expt(-(x0.^2)/2)/sqrt(2*pi)./s0;
i0=p'f0*p;

for ri (1,rn,1);
  r=rs[ri];
  h1=hs*GaussRuleThumb(75,r,sd);
  for hi (1,hn,1);
    h=h1[hi];
    k=GaussCF(x*h,r); 
    intg1a[_m,ri,hi]=((k.*fs)'w)/pi;
    intg2a[_m,ri,hi]=(((k.^2).*fs)'w)/pi;
    h2=h.^2;
    s1=sqrt(sig2+h2);
    s2=sqrt(sig2+h2*2);
    x1=(mu-mu')./s1;
    x2=(mu-mu')./s2;
    f1=(expt(-(x1.^2)/2)./s1)/sqrt(2*pi);
    f2=(expt(-(x2.^2)/2)./s2)/sqrt(2*pi);
    i1=0;i2=0;
    for i (0,r-1,1);a=i;
      if i==0; pr=1; else;  pr=prodc(seqa(.5,1,a)); endif;
      i1=i1+(p'((f1.*HyperGeometric1F1(-a,.5,(x1.^2)/2,1000,1e-16).*pr).*((h./s1).^(2*a)))*p)/(a!);
      for j (0,r-1,1);b=j;
        ij=i+j;
        if ij==0; pr=1; else;  pr=prodc(seqa(.5,1,ij)); endif;
        i2=i2+(p'((f2.*HyperGeometric1F1(-ij,.5,(x2.^2)/2,1000,1e-16).*pr).*((h./s2).^(2*ij)))*p)/(a!)/(b!);
      endfor;
    endfor;
    intg1[_m,ri,hi]=i1;
    intg2[_m,ri,hi]=i2;
  endfor;
endfor;
endfor;


@ Polynomial Kernels @

hn=50;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

intp1=arrayinit(_mmax|rn|hn,ms);
intp2=arrayinit(_mmax|rn|hn,ms);
intp1a=arrayinit(_mmax|rn|hn,ms);
intp2a=arrayinit(_mmax|rn|hn,ms);

for _m (1,_mmax,1);

bb=bbs[_m];
x=((-rev(xx))|xx)*(bb/2)+bb/2;
w=(rev(ww)|ww)*(bb/2);

{p,mu,sig} = models(_m);
q=rows(p);
fs=abs((expt(-((x*(sig')).^2)/2).*expt(x*sqrt(-1)*(mu')))*p).^2;
sd=mn_sd(_m);
sig2=(sig.^2)+(sig.^2)';
s0=sqrt(sig2);
x0=(mu-mu')./s0;
f0=expt(-(x0.^2)/2)/sqrt(2*pi)./s0;
i0=p'f0*p;

x2=(x0.^2)/2;
for ri (1,rn,1);
    r=rs[ri];
    h1=hs*PolyRuleThumb(75,r,s,sd);
    h2=-((h1.^2)/2)';
    for hi (1,hn,1);
      h=h1[hi];
      k=PolyCF(x*h,r,s); 
      intp1a[_m,ri,hi]=((k.*fs)'w)/pi;
      intp2a[_m,ri,hi]=(((k.^2).*fs)'w)/pi;
    endfor;
endfor;

rmax=rs[rn];
sr=seqa(0,1,rmax)';
alpha=seqa(s+.5,2,rmax).*(gamma(seqa(s+.5,1,rmax))./(seqa(0,1,rmax))!);
jmax=2000;
cs=zeros(jmax,rmax);
for m (0,rmax-1,1);
  if m==0;
    c=1/gamma(s+1.5);
  else;
    c=prodc(seqa(.5,1,m))/gamma(s+2*m+1.5)./((sig2).^m);
  endif;
  cc=p'(c.*HyperGeometric1F1(-m,.5,x2,jmax,1e-16).*f0)*p;
  for j (1,jmax-1,1);
    if maxc(abs(c)) < 1e-300; break; endif;
    c=-c.*(m+j-.5)./j./(s+m*2+.5+j)./(sig2); 
    cc=cc|(p'(c.*HyperGeometric1F1(-m-j,.5,x2,jmax,1e-16).*f0)*p);
  endfor;
  cs[1:rows(cc),m+1]=cc;
endfor;

for ri (1,rn,1);
  r=rs[ri];
  h1=hs*PolyRuleThumb(75,r,s,sd);
  h2=(h1.^2)/2;
  br=cs[jmax,.];
  for j (jmax-1,1,-1);
    br=br.*h2+cs[j,.];
  endfor;
  b = (br.*(h2.^sr)).*(alpha');
  b=cumsumc(b')';
  int1=b[.,r];
  int1a=arraytomat(intp1a[_m,ri,.])';
  int1 = missex(int1, (int1.< (int1a*.9))+(int1.> (int1a*1.1)));
  intp1[_m,ri,.]=int1';
endfor;

a=zeros(hn,rmax^2);
ab=zeros(rmax,rmax);
cs=zeros(jmax,rmax^2);
for m (0,rmax-1,1);
for l (0,rmax-1,1);
  ab[m+1,l+1]=1+maxc(m|l);
  ml=m+l;
  if ml==0;
    c=1/(gamma(s+1.5)^2);
  else;
    c=prodc(seqa(.5,1,ml))/gamma(s+2*m+1.5)/gamma(s+2*l+1.5)./((sig2).^ml);
  endif;
  cc=p'(c.*HyperGeometric1F1(-ml,.5,x2,jmax,1e-16).*f0)*p;
  for j (1,jmax-1,1);
    if maxc(abs(c)) < 1e-300; break; endif;
    c=-c.*(ml+j-.5)./j.*(2*s+2*ml+2*j+1).*(2*s+2*ml+2*j)./(2*s+2*ml+j+1)./(j+s+2*l+.5)./(j+s+2*m+.5)./(sig2); 
    cc=cc|(p'(c.*HyperGeometric1F1(-ml-j,.5,x2,jmax,1e-16).*f0)*p);
  endfor;
  cs[1:rows(cc),m*rmax+l+1]=cc;
endfor;
endfor;
ab=vec(ab)';

for ri (1,rn,1);
  r=rs[ri];
  h1=hs*PolyRuleThumb(75,r,s,sd);
  h2=(h1.^2)/2;
  ar=cs[jmax,.];
  for j (jmax-1,1,-1);
    ar=ar.*h2+cs[j,.];
  endfor;
  ar = ar.*(h2.^(vec(sr+sr')')).*((alpha.*.alpha)');
  int2=sumc((ar.*(ab.<=r))');
  int2a=arraytomat(intp2a[_m,ri,.])';
  int2 = missex(int2, (int2.< (int2a*.9))+(int2.> (int2a*1.1)));
  intp2[_m,ri,.]=int2';
endfor;

endfor;


@ Trapezoidal Kernels @

hn=60;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

intt1=arrayinit(_mmax|2|hn,ms);
intt2=zeros(_mmax,hn)+ms;
intt1a=arrayinit(_mmax|2|hn,ms);
intt2a=zeros(_mmax,hn)+ms;

for _m (1,_mmax,1);

bb=bbs[_m];
x=((-rev(xx))|xx)*(bb/2)+bb/2;
w=(rev(ww)|ww)*(bb/2);

{p,mu,sig} = models(_m);
q=rows(p);
fs=abs((expt(-((x*(sig')).^2)/2).*expt(x*sqrt(-1)*(mu')))*p).^2;
sd=mn_sd(_m);
sig2=(sig.^2)+(sig.^2)';
s0=sqrt(sig2);
x0=(mu-mu')./s0;
f0=expt(-(x0.^2)/2)/sqrt(2*pi)./s0;
i0=p'f0*p;

h0=TrapRuleThumb(75,sd);
h1=hs*h0;
for hi (1,hn,1);
  h=h1[hi];
  k=((x*h).<=1); 
  intt1a[_m,1,hi]=((k.*fs)'w)/pi;
  intt1[_m,1,hi]=TrapC1(1./h,_m);
  k=TrapCF(x*h);
  intt1a[_m,2,hi]=((k.*fs)'w)/pi;
  intt2a[_m,hi]=(((k.^2).*fs)'w)/pi;
  intt1[_m,2,hi]=2*TrapC1(2./h,_m)-h.*TrapC2(2./h,_m)
         -TrapC1(1./h,_m)+h.*TrapC2(1./h,_m);
  intt2[_m,hi]=4*TrapC1(2./h,_m)-4*h.*TrapC2(2./h,_m)+(h.^2).*TrapC3(2./h,_m)
           -3*TrapC1(1./h,_m)+4*h.*TrapC2(1./h,_m)-(h.^2).*TrapC3(1./h,_m);
endfor;

endfor;

saveall imse2;

/*****************************************************/


@ Gaussian Kernel Roughness @
proc GaussRoughness(r);
local j,a,rg;
  j=seqa(0,1,r);
  a=1;
  if r>1;a=a|(1./cumprodc(seqa(2,2,r-1)));
  endif;
  rg=(a'gamma(j+j'+.5)*a)/2/pi;
retp(rg);
endp;

@ Rule-of-Thumb for Gaussian kernels @
proc GaussRuleThumb(n,r,sd);
local h,c,rf,m2;
  m2=(2^(2*r))*(gamma(r+.5)^2)/pi;
  rf=gamma(2*r+.5)/2/pi;
  c=GaussRoughness(r)/rf/m2/4/r*(((2*r)!).^2);
  h=(c/n)^(1/(1+4*r));
  h=h*sd;
retp(h);
endp;

@ Characteristic Function of Gaussian Kernel @
proc GaussCF(x,r);
local t,tt,f,ff;
    t=vecr(x);
    tt=((t.^2)/2);
    f=expt(-tt);
    if r>1;
      f=f.*(1+sumc(cumprodc((1./seqa(1,1,r-1)).*(tt'))));
    endif;
    ff=reshape(f,rows(x),cols(x));
retp(ff);
endp;

/******************************************************/


proc PolyRoughness(r,s);
local j,a,jk,jkf,b,rg,jk1,bb;
  a=1;
  if r>1;
    a=a|cumprodc((s-.5)./seqa(1,1,r-1)+1);
  endif;
  a=(a.*seqa(s+.5,2,r))*gamma(s+.5);
  j=seqa(0,1,r);
  jk=(j-j'+s);
  jk1=(jk.>=0);
  jkf=(jk.*jk1)!;
  bb=gamma(j+j'+2*s+1.5)./gamma(j+j'+.5);
  b=(jk1.*(jk1'))./(jkf.*(jkf'))./bb; 
  rg=(a'b*a)*((2*s)!)/pi;
retp(rg);
endp;


@ Rule-of-Thumb for Polynomial kernels @
proc PolyRuleThumb(n,r,s,sd);
local h,c,rf,m2;
  m2=((gamma(r+s+.5)*gamma(r+.5)/gamma(2*r+s+.5)).^2)/pi;
  rf=gamma(2*r+.5)/2/pi;
  c=PolyRoughness(r,s)/rf/m2/4/r*(((2*r)!).^2);
  h=(c/n)^(1/(1+4*r));
  h=h*sd;
retp(h);
endp;


@ Characteristic Function of Polynomial Kernel @
proc PolyCF(x,r,s);
local tk,tt,f,ti,b,a,t,ff;
    tk=vecr(x);
    t=abs(tk);
    ti=(t.<= 1e-100);
    tt=t+ti;
    a=1;
    if r>1;
      a=a|cumprodc((s-.5)./seqa(1,1,r-1)+1);
    endif;
    a=(a.*seqa(s+.5,2,r))*gamma(s+.5);
    b=SphericalBessel(tt,s+2*r-2);
    f=b[.,seqa(s,2,r)]*a;
    f=f.*((2./tt).^s);
    f=f*(2/sqrt(pi));
    f=f.*(1-ti)+ti;    
    f=missrv(f,1);
    ff=reshape(f,rows(x),cols(x));
retp(ff);
endp;

/******************************************************/

@ Trapezoidal Kernel  @
fn TrapRuleThumb(n,sd)=sd/sqrt(ln((n-1)));
fn TrapCF(t)=(2-t).*(t.<=2)-(1-t).*(t.<=1);


proc TrapC1(theta,_m);
local jmax,p,mu,sig,y,x2,ya,c,cc,j,p2,s2,g;
  jmax=100;
  {p,mu,sig} = models(_m);
  s2=vec((sig.^2)+(sig.^2)');
  y=-(((vec(mu-mu')).^2)./s2)/2;
  s2=s2/2;
  x2=(theta.^2).*s2;
  p2=(p.*.p)./sqrt(s2);
  c=sigamma(.5,x2);
  cc=c; ya=1;
  for j (1,jmax,1);
    ya=ya.*y/j;    
    c=sigamma(.5+j,x2).*ya;
    cc=cc+c;
    if maxc(abs(c)) < 1e-16; break; endif;
  endfor;
  g=(cc'p2)/(2*sqrt(pi));
retp(g);
endp;


proc TrapC2(theta,_m);
local jmax,p,mu,sig,y,x2,ya,c,cc,j,p2,s2,g;
  jmax=100;
  {p,mu,sig} = models(_m);
  s2=vec((sig.^2)+(sig.^2)');
  y=-(((vec(mu-mu')).^2)./s2)/2;
  s2=s2/2;
  x2=(theta.^2).*s2;
  p2=(p.*.p)./s2;
  c=sigamma(1,x2);
  cc=c; ya=1;
  for j (1,jmax,1);
    ya=ya.*y/(j-.5);    
    c=sigamma(1+j,x2).*ya;
    cc=cc+c;
    if maxc(abs(c)) < 1e-16; break; endif;
  endfor;
  g=cc'p2;
  g=g/(2*pi);
retp(g);
endp;


proc TrapC3(theta,_m);
local jmax,p,mu,sig,y,x2,ya,c,cc,j,p2,s2,g;
  jmax=100;
  {p,mu,sig} = models(_m);
  s2=vec((sig.^2)+(sig.^2)');
  y=-(((vec(mu-mu')).^2)./s2)/2;
  s2=s2/2;
  x2=(theta.^2).*s2;
  p2=(p.*.p)./(s2.^ 1.5);
  c=sigamma(1.5,x2);
  cc=c; ya=1;
  for j (1,jmax,1);
    ya=ya.*y*((j+.5)/((j-.5)*j));    
    c=(sigamma(1.5+j,x2).*ya);
    cc=cc+c;
    if maxc(abs(c)) < 1e-16; break; endif;
  endfor;
  g=cc'p2;
  g=g/(4*sqrt(pi));
retp(g);
endp;


/******************************************************/


@ Standard deviation of Mixture-Normal Densities @
proc mn_sd(_m);
local p,mu,sig,m,m2,s;
  {p,mu,sig} = models(_m);
  m=p'mu;
  m2=p'((sig.^2)+mu.^2);
  s=sqrt(m2-m.^2);
retp(s);
endp;

@ Mixture-Normal Characteristic Function @
proc MNCF(_m,x);
local p,mu,sig,f1,f2,f,i;
  {p,mu,sig} = models(_m);
  i=sqrt(-1);
  f=(expt(-((x*(sig')).^2)/2).*expt(x*i*(mu')))*p;
retp(f);
endp;


@ Marron-Wand Mixture-Normal Parameterization @
proc (3) = models(_m);
local p,mu,sig;
if _m==1;
  p=1;mu=0; sig=1;
elseif _m==2;
  p= .2|.2|.6;
  mu= 0|.5|13/12;
  sig = 1|2/3|5/9;
elseif _m==3;
  p=ones(8,1)*(1/8);
  sig=(2/3).^seqa(0,1,8);
  mu=(sig-1)*3;
elseif _m==4;
  p=(2/3)|(1/3);
  mu=0|0;
  sig=1|.1;
elseif _m==5;
  p=.1|.9;
  mu=0|0;
  sig=1|.1;
elseif _m==6;
  p=.5|.5;
  mu=-1|1;
  sig=2/3|2/3;
elseif _m==7;
  p=.5|.5;
  mu=-1.5|1.5;
  sig=1/2|1/2;
elseif _m==8;
  p=.75|.25;
  mu=0|1.5;
  sig=1|1/3;
elseif _m==9;
  p=9/20|9/20|.1;
  mu=-1.2|1.2|0;
  sig=.6|.6|.25;
elseif _m==10;
  p=.5|.1|.1|.1|.1|.1;
  mu=0|seqa(-1,.5,5);
  sig=1|.1|.1|.1|.1|.1;
elseif _m==11;
  p=.49|.49|(ones(7,1)/350);
  mu=-1|1|seqa(-1.5,.5,7);
  sig=(2/3)|(2/3)|(ones(7,1)/100);
elseif _m==12;
  p=.5|((2.^seqa(3,-1,5))/31);
  mu=0|seqa(-1.5,1,5);
  sig=1|((2.^seqa(2,-1,5))/10);
elseif _m==13;
  p=.46|.46|(1/300)|(1/300)|(1/300)|(7/300)|(7/300)|(7/300);
  mu=-1|1|-.5|-1|-1.5|.5|1|1.5;
  sig=(2/3)|(2/3)|.01|.01|.01|.07|.07|.07;
elseif _m==14;
  p=(2.^seqa(5,-1,6))/63;
  mu=(65-(.5.^seqa(0,1,6))*96)/21;
  sig=(.5.^seqa(0,1,6))*32/63;
elseif _m==15;
  p=(2/7)|(2/7)|(2/7)|(1/21)|(1/21)|(1/21);
  mu=-(15/7)|-(3/7)|(9/7)|(16/7)|(18/7)|(20/7);
  sig=p;
endif;
retp(p,mu,sig);
endp;

/******************************************************/

@ trimmed exponential : Prevents Underflow @
proc expt(x);
local x1,x2,f;
  x1=(x.< -700);
  x2=(x.> 700);
  f=exp(x.*(1-x1-x2))-x1+x2*exp(700);
retp(f);
endp;

@ Natural log of factorial using Stirling's Approximation @
@ Source: Magnus, Oberhettinger, Soni (1966), p. 12 @
proc LnFactorial(x);
local d1,x1,xt,gb,np,n,nn,ni,m,b,g,xx,r,c;
    r=rows(x);c=cols(x);
    xx=vec(x);
    d1=(xx.<= -1);	
    x1=xx.*(1-d1)+1;			@ adds 1 to convert factorial to Gamma 	@
    xt=x1+2; 				@ adds 2 for accuracy 	@
    gb=0.5*ln(2*pi)+(xt-0.5).*ln(xt)-xt;
    np=3;				@ number of polynomial terms @
    if np>0;
      n=seqa(2,2,np);
      nn=n+1-n';ni=nn.>=0;nn=(nn.*ni)!;
      m=((((n+1)!)./((n!)'))./nn).*ni;
      b=inv(m)*((n-1)/2);		@ Bernouli numbers, 2 thru 2*np @
      gb=gb+(xt.^(-n'+1))*(b./n./(n-1));
    endif;
    g=(gb-ln(x1.*(x1+1))).*(1-d1);
retp(reshape(g,r,c));
endp;


@ Incomplete Gamma Function @
@ Based on Fortran code in Zhang and Jin (1996) @
proc igamma(a,xx);
local rx,cx,gg,ri,ci,x,xam,g,r,j;
  rx=rows(xx);cx=cols(xx);
  if (a .> 170) .or (a .<= 0);
    gg=zeros(rx,cx)+miss(0,0);
  else;
    gg=zeros(rx,cx);
    for ri (1,rx,1);
    for ci (1,cx,1);
      x=xx[ri,ci];
      xam=-x+ln(x).*a;
      if (xam .> 700);
        g=miss(0,0);
      elseif (x .<= a+1);
        g=1./a;
        r=g;
        for j (1,100,1);
          r=r.*x./(a+j);
          g=g+r;
          if abs(r./g) .< 1e-16; break; endif;
        endfor;
        g=g.*exp(xam);
      else;
        g=0;
        for j (60,1,-1);
          g=(j-a)./(1+j./(x+g));
        endfor;
        g=gamma(a)-exp(xam)./(x+g);
      endif;
      gg[ri,ci]=g;
    endfor;
    endfor;
  endif;
retp(gg);
endp;


@ Scaled Incomplete Gamma Function @
@ Ratio of Incomplete Gamma to Gamma @
@ Based on Fortran code in Zhang and Jin (1996) @
proc sigamma(a,xx);
local rx,cx,gg,ri,ci,x,xam,g,r,j;
  rx=rows(xx);cx=cols(xx);
  if (a .<= 0);
    gg=zeros(rx,cx)+miss(0,0);
  else;
    gg=zeros(rx,cx);
    for ri (1,rx,1);
    for ci (1,cx,1);
      x=xx[ri,ci];
      xam=-x+ln(x).*a-LnFactorial(a-1);
      if (xam .> 700);
        g=miss(0,0);
      elseif (x .<= a+1);
        g=1./a;
        r=g;
        for j (1,100,1);
          r=r.*x./(a+j);
          g=g+r;
          if abs(r./g) .< 1e-16; break; endif;
        endfor;
        g=g.*exp(xam);
      else;
        g=0;
        for j (60,1,-1);
          g=(j-a)./(1+j./(x+g));
        endfor;
        g=1-exp(xam)./(x+g);
      endif;
      gg[ri,ci]=g;
    endfor;
    endfor;
  endif;
retp(gg);
endp;



/******************************************************
SphericalBessel

Format: b=SphericalBessel(x,n)

Inputs:	
	x	rx1, points of evaluation
		nonnegative reals
	n	nonnegative integer
		order of spherical Bessel function

Output:
	b	rxm
		Spherical Bessel functions of the first kind
		Orders 1,2,...,m


********************************************************/

proc SphericalBessel(x,m);
local xi,x1,b0,b1,b2,b,mm,x2,a,xx,x22;
  x1=x+(x.<= .01);
  b0=(sin(x1)./x1);
  b1=((b0-cos(x1))./x1);
  b=b1;
  if m>1;
  @ forward recursion @
    for j (2,m,1);
      b2=-b0+(b1./x1)*(2*j-1);
      b=b~b2;
      b0=b1;b1=b2;
    endfor;
  endif;
  @ series method @
  x2=x/2;
  x22=-(x2.^2);
  for mm (1,m,1);
    for xi (1,rows(x),1);
      xx=x2[xi];
      if (xx .< (mm*.375));
        a=(xx^mm)/prodc(seqa(.5,1,mm+1))/2;
        b[xi,mm]=HyperGeometric0f1(1.5+mm,x22[xi],100,1e-8).*a;
      endif;
    endfor;
  endfor;
retp(b);
endp;


proc HyperGeometric0F1(c,x,max,tol);
local d,f,j;
  d=1; f=1;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x)./((c+j-1)*j);
    f=f+d;
  endfor;
  f=missex(f,(abs(d./f) .> tol));
retp(f);
endp;

proc HyperGeometric1f1(a,c,x,max,tol);
local d,f,j,it;
  it=0;
  d=1; f=1;
  if prodc((a.==round(a)).*(a.<0))==1;
    it=1;
    max=maxc(vec(-a));
  endif;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x).*((a+j-1)./(c+j-1)/j);
    f=f+d;
  endfor;
  if it .ne 1;
    f=missex(f,(abs(d./f) .> tol));
  endif;
retp(f);
endp;


proc HyperGeometric2f1(a,b,c,x,max,tol);
local d,f,j,it;
  it=0;
  d=1; f=1;
  if prodc((a.==round(a)).*(a.<0))==1;
    it=1;
    max=maxc(vec(-a));
  endif;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x).*((a+j-1).*(b+j-1)./(c+j-1)/j);
    f=f+d;
  endfor;
  if it .ne 1;
    f=missex(f,(abs(d./f) .> tol));
  endif;
retp(f);
endp;



