/***************************************************************   
IMSE2-P.PRG

written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
www.ssc.wisc.edu/~bhansen
bhansen@ssc.wisc.edu

This Gauss program plots numerical and analytic integrals for evaluation of MISE
as described in the  paper
"Exact Mean Integrated Squared Error of Higher-Order Kernel Estimators"
and creates the figures displayed in the appendix.

The program uses the Gauss file "imse2.gcg" which is created by first
running the program "imse2-c.prg".

*****************************************************************/
use imse2;

@ Gaussian Kernels @

hn=300;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

ri=1; do while ri<=rn;

graphset;
pqgwin many;
xtics(0,hmax,.5,50);
_protate=1;
_pcolor=15;
let _pltype= 1 1 4 4;
fonts("complex");
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_ptitlht=.25;

r=rs[ri];
begwind;
window(4,3,0); 
makewind(6.855,9,0,0,1);
setwind(1);
for m (1,6,1);
  i1=arraytomat(intg1[m,ri,.])';
  i2=arraytomat(intg2[m,ri,.])';
  i1a=arraytomat(intg1a[m,ri,.])';
  i2a=arraytomat(intg2a[m,ri,.])';
  ii=i1~i2~i1a~i2a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", r = " $+ ftocv(r,1,0);
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

ri=ri+1;
r=rs[ri];
for m (1,6,1);
  i1=arraytomat(intg1[m,ri,.])';
  i2=arraytomat(intg2[m,ri,.])';
  i1a=arraytomat(intg1a[m,ri,.])';
  i2a=arraytomat(intg2a[m,ri,.])';
  ii=i1~i2~i1a~i2a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", r = " $+ ftocv(r,1,0);
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

ylabel("");
xlabel("");
title("");
_paxes = 0;
let _pmsgctl[1,7] = 
1.25 8.85 .125 0 2 15 0 ;
_pmsgstr = "Numerical Integration Versus Series Expansion, Gaussian Kernel";
draw;

graphprt("-p");
endwind;

ri=ri+1;endo;


@ Polynomial Kernels @

hn=50;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

ri=1; do while ri<=rn;

graphset;
pqgwin many;
xtics(0,hmax,.2,10);
_protate=1;
_pcolor=15;
let _pltype= 1 1 4 4;
fonts("complex");
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_ptitlht=.25;

r=rs[ri];
begwind;
window(4,3,0); 
makewind(6.855,9,0,0,1);
setwind(1);
for m (1,6,1);
  i1=arraytomat(intp1[m,ri,.])';
  i2=arraytomat(intp2[m,ri,.])';
  i1a=arraytomat(intp1a[m,ri,.])';
  i2a=arraytomat(intp2a[m,ri,.])';
  ii=i1~i2~i1a~i2a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", r = " $+ ftocv(r,1,0) $+ " , s = " $+ ftocv(s,1,0);
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

ri=ri+1;
r=rs[ri];
for m (1,6,1);
  i1=arraytomat(intp1[m,ri,.])';
  i2=arraytomat(intp2[m,ri,.])';
  i1a=arraytomat(intp1a[m,ri,.])';
  i2a=arraytomat(intp2a[m,ri,.])';
  ii=i1~i2~i1a~i2a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", r = " $+ ftocv(r,1,0) $+ " , s = " $+ ftocv(s,1,0);
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

ylabel("");
xlabel("");
title("");
_paxes = 0;
let _pmsgctl[1,7] = 
1.25 8.85 .125 0 2 15 0 ;
_pmsgstr = "Numerical Integration Versus Series Expansion, Polynomial Kernel";
draw;

graphprt("-p");
endwind;

ri=ri+1;endo;

@ Trapezoidal Kernels @

hn=60;
inc=1/hn;
hmax=3;
hs=seqa(inc,inc,hn)*hmax;

graphset;
pqgwin many;
xtics(0,hmax,.5,10);
_protate=1;
_pcolor=15;
fonts("complex");
_plwidth=3;
_paxht=.2;
_pnumht=.15;
_ptitlht=.25;

let _pltype= 1 4;
begwind;
window(4,3,0); 
makewind(6.855,9,0,0,1);
setwind(1);
for m (1,6,1);
  i1=arraytomat(intt1[m,1,.])';
  i1a=arraytomat(intt1a[m,1,.])';
  ii=i1~i1a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", Dirichlet Kernel";
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

let _pltype= 1 1 4 4;
for m (1,6,1);
  i1=arraytomat(intt1[m,2,.])';
  i2=intt2[m,.]';
  i1a=arraytomat(intt1a[m,2,.])';
  i2a=intt2a[m,.]';
  ii=i1~i2~i1a~i2a;
  tit = "Model " $+ ftocv(m,1,0) $+ ", Trapezoid Kernel";
  title(tit);
  xy(hs,ii);
  nextwind;
endfor;

ylabel("");
xlabel("");
title("");
_paxes = 0;
let _pmsgctl[1,7] = 
1.25 8.85 .125 0 2 15 0 ;
_pmsgstr = "Numerical Integration Versus Series Expansion, Flat-Top Kernels";
draw;

graphprt("-p");
endwind;




/*****************************************************/
