/***************************************************************   
PLOT.PRG

written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
www.ssc.wisc.edu/~bhansen
bhansen@ssc.wisc.edu

This Gauss program plots the kernel functions and their characteristic functions
as described in the paper
"Exact Mean Integrated Squared Error of Higher-Order Kernel Estimators"
and creates Figure 1 for that paper

*****************************************************************/


b=5;
xn=1000;
x=seqa(-b,2*b/xn,xn+1);
t=seqa(0,.006,xn+1);

kp=zeros(xn+1,4);
cp=zeros(xn+1,4);
kg=zeros(xn+1,3);
cg=zeros(xn+1,3);
r=2;
h=PolyRoughness(r,8);
kp[.,1]=PolyKernel(x/h,r,8)/h;
cp[.,1]=PolyCF(t.*h,r,8);
h=GaussRoughness(r);
kg[.,1]=GaussKernel(x/h,r)/h;
cg[.,1]=GaussCF(t.*h,r);
r=5;
h=PolyRoughness(r,8);
kp[.,2]=PolyKernel(x/h,r,8)/h;
cp[.,2]=PolyCF(t.*h,r,8);
h=GaussRoughness(r);
kg[.,2]=GaussKernel(x/h,r)/h;
cg[.,2]=GaussCF(t.*h,r);
r=10;
h=PolyRoughness(r,8);
kp[.,3]=PolyKernel(x/h,r,8)/h;
cp[.,3]=PolyCF(t.*h,r,8);
h=GaussRoughness(r);
kg[.,3]=GaussKernel(x/h,r)/h;
cg[.,3]=GaussCF(t.*h,r);
h=4/3/pi;
kp[.,4]=TrapKernel(x/h)/h;
cp[.,4]=TrapCF(t.*h);



graphset;
pqgwin many;
_pcolor=15;
fonts("complex simgrma");
_paxes=1;
_protate=1;
_plwidth=4;
_pypmax=1;
_pdate="";
_paxht=.2;
_pnumht=.15;
_ptitlht=.35;
let _pltype= 2 4 3 6;
_plegstr1="K]4,8[\000K]10,8[\000K]20,8[\000\202l\201]2[";
_plegstr2="G]4[\000G]10[\000G]20";
xtics(-5,5,1,10);
ytics(-.2,1.2,.2,20);


begwind;
window(4,1,0);
  let _plegctl= 1 8 4.25 .75;
  _plegstr=_plegstr1;
  setwind(1);
  title("Figure 1:  Polynomial and Trapezoid Kernel Functions");
  xy(x,kp);
nextwind;
  let _plegctl= 1 8 4.35 .827;
  _plegstr=_plegstr2;
  title("Gaussian Kernel Functions");
  xy(x,kg);
nextwind;
  let _plegctl= 1 8 5.55 .62;
  xtics(0,6,1,10);
  ytics(-.2,1,.2,20);
  _plegstr=_plegstr1;
  title("Polynomial and Trapezoid Characteristic Functions");
  xy(t,cp);
nextwind;
  let _plegctl= 1 8 5.59 .69;
  _plegstr=_plegstr2;
  title("Gaussian Characteristic Functions");
  xy(t,cg);
endwind;


/*****************************************************/


@ Gaussian Kernel Roughness @
proc GaussRoughness(r);
local j,a,rg;
  j=seqa(0,1,r);
  a=1;
  if r>1;a=a|(1./cumprodc(seqa(2,2,r-1)));
  endif;
  rg=(a'gamma(j+j'+.5)*a)/2/pi;
retp(rg);
endp;

@ Characteristic Function of Gaussian Kernel @
proc GaussCF(x,r);
local t,tt,f,ff;
    t=vecr(x);
    tt=((t.^2)/2);
    f=expt(-tt);
    if r>1;
      f=f.*(1+sumc(cumprodc((1./seqa(1,1,r-1)).*(tt'))));
    endif;
    ff=reshape(f,rows(x),cols(x));
retp(ff);
endp;

@ Gaussian Kernel function @
proc GaussKernel(x,r);
local k,x2,f;
    x2=(x.^2)/2;
    k=expt(-x2).*HyperGeometric1f1(-r+1,1.5,x2,10000,1e-16);
    k=k./sqrt(2*pi);
    if r>1;
      k=k*prodc(seqa(1.5,1,r-1)./seqa(1,1,r-1));
    endif;
retp(k);
endp;



/******************************************************/

proc PolyRoughness(r,s);
local j,a,jk,jkf,b,rg,jk1,bb;
  a=1;
  if r>1;
    a=a|cumprodc((s-.5)./seqa(1,1,r-1)+1);
  endif;
  a=(a.*seqa(s+.5,2,r))*gamma(s+.5);
  j=seqa(0,1,r);
  jk=(j-j'+s);
  jk1=(jk.>=0);
  jkf=(jk.*jk1)!;
  bb=gamma(j+j'+2*s+1.5)./gamma(j+j'+.5);
  b=(jk1.*(jk1'))./(jkf.*(jkf'))./bb; 
  rg=(a'b*a)*((2*s)!)/pi;
retp(rg);
endp;


@ Characteristic Function of Polynomial Kernel @
proc PolyCF(x,r,s);
local tk,tt,f,ti,b,a,t,ff;
    tk=vecr(x);
    t=abs(tk);
    ti=(t.<= 1e-100);
    tt=t+ti;
    a=1;
    if r>1;
      a=a|cumprodc((s-.5)./seqa(1,1,r-1)+1);
    endif;
    a=(a.*seqa(s+.5,2,r))*gamma(s+.5);
    b=SphericalBessel(tt,s+2*r-2);
    f=b[.,seqa(s,2,r)]*a;
    f=f.*((2./tt).^s);
    f=f*(2/sqrt(pi));
    f=f.*(1-ti)+ti;    
    f=missrv(f,1);
    ff=reshape(f,rows(x),cols(x));
retp(ff);
endp;

@ Polynomial Kernel function @
proc PolyKernel(x,r,s);
local ax,x2,c,k;
  ax=(abs(x).<=1);
  x2=(x.*ax).^2;
  c=prodc(1./seqa(2,2,s+r-1)+1)/2;
  if r>1;
    c=c*prodc(1./seqa(2,2,r-1)+1);
  endif;
  k=c*((1-x2).^s).*HyperGeometric2f1(-r+1,r+s+.5,1.5,x2,10000,1e-16).*ax;
retp(k);
endp;


/******************************************************/


@ Trapezoidal Kernel  @
fn TrapRuleThumb(n,sd)=sd/sqrt(ln((n-1)));
fn TrapCF(t)=(2-t).*(t.<=2)-(1-t).*(t.<=1);

proc TrapKernel(x);
local xi,xh,k;
    xi=(x .== 0);
    xh=(x+xi).^2;
    k=(((((sin(x)).^2)-((sin(x/2)).^2))./xh)+xi*.75)*(2/pi);
retp(k);
endp;

proc DirichletKernel(x);
local xi,k;
    xi=(x .== 0);
    k=(sin(x)./(x+xi) + xi)/pi;
retp(k);
endp;


/******************************************************/

@ trimmed exponential : Prevents Underflow @
proc expt(x);
local x1,x2,f;
  x1=(x.< -700);
  x2=(x.> 700);
  f=exp(x.*(1-x1-x2))-x1+x2*exp(700);
retp(f);
endp;


/******************************************************
SphericalBessel

Format: b=SphericalBessel(x,n)

Inputs:	
	x	rx1, points of evaluation
		nonnegative reals
	n	nonnegative integer
		order of spherical Bessel function

Output:
	b	rxm
		Spherical Bessel functions of the first kind
		Orders 1,2,...,m


********************************************************/
proc SphericalBessel(x,m);
local xi,x1,b0,b1,b2,b,mm,x2,a,xx,x22;
  x1=x+(x.<= .01);
  b0=(sin(x1)./x1);
  b1=((b0-cos(x1))./x1);
  b=b1;
  if m>1;
  @ forward recursion @
    for j (2,m,1);
      b2=-b0+(b1./x1)*(2*j-1);
      b=b~b2;
      b0=b1;b1=b2;
    endfor;
  endif;
  @ series method @
  x2=x/2;
  x22=-(x2.^2);
  for mm (1,m,1);
    for xi (1,rows(x),1);
      xx=x2[xi];
      if (xx .< (mm*.375));
        a=(xx^mm)/prodc(seqa(.5,1,mm+1))/2;
        b[xi,mm]=HyperGeometric0f1(1.5+mm,x22[xi],100,1e-8).*a;
      endif;
    endfor;
  endfor;
retp(b);
endp;


proc HyperGeometric0F1(c,x,max,tol);
local d,f,j;
  d=1; f=1;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x)./((c+j-1)*j);
    f=f+d;
  endfor;
  f=missex(f,(abs(d./f) .> tol));
retp(f);
endp;

proc HyperGeometric1f1(a,c,x,max,tol);
local d,f,j,it;
  it=0;
  d=1; f=1;
  if prodc((a.==round(a)).*(a.<0))==1;
    it=1;
    max=maxc(vec(-a));
  endif;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x).*((a+j-1)./(c+j-1)/j);
    f=f+d;
  endfor;
  if it .ne 1;
    f=missex(f,(abs(d./f) .> tol));
  endif;
retp(f);
endp;


proc HyperGeometric2f1(a,b,c,x,max,tol);
local d,f,j,it;
  it=0;
  d=1; f=1;
  if prodc((a.==round(a)).*(a.<0))==1;
    it=1;
    max=maxc(vec(-a));
  endif;
  for j (1,max,1);
    if maxc(maxc(missrv(abs(d./f),0))) < tol; break; endif;
    d=(d.*x).*((a+j-1).*(b+j-1)./(c+j-1)/j);
    f=f+d;
  endfor;
  if it .ne 1;
    f=missex(f,(abs(d./f) .> tol));
  endif;
retp(f);
endp;



