/***************************************************************   
REGRET.PRG

written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
www.ssc.wisc.edu/~bhansen
bhansen@ssc.wisc.edu

This Gauss program calculates the minimax regret of the kernel classes
as described in the paper
"Exact Mean Integrated Squared Error of Higher-Order Kernel Estimators"
and creates Figure 5 for that paper.

The program uses the Gauss file "imse1.gcg" which is created by first
running the program "imse1-c.prg".

If the switch "minimax_r" is set to one, the program prints the minimax
optimal kernel order r by sample size,

*****************************************************************/
use imse1;

minimax_r==0;		@ set to one to print minimax kernel orders r @

s=8;si=3;

if minimax_r==0;
  nn=40;
  ns=seqa(25,25,nn);
elseif minimax_r==1;
  nn=4000;
  ns=seqa(10,1,nn);
endif;

/*****************************************************************/

regret=zeros(nn,5);
regretr=zeros(nn,2);

for ni (1,nn,1);
n=ns[ni];

mg=zeros(_mmax,rn);
mp=zeros(_mmax,rn);
md=zeros(_mmax,1);
mt=zeros(_mmax,1);

for _m (1,_mmax,1);

{p,mu,sig} = models(_m);
sd=mn_sd(_m);
s0=sqrt((sig.^2)+(sig.^2)');
x0=(mu-mu')./s0;
f0=expt(-(x0.^2)/2)/sqrt(2*pi)./s0;
i0=p'f0*p;

m1=zeros(rn,1);
m2=zeros(rn,1);

for ri (1,rn,1);
  r=rs[ri];
  h0=GaussRuleThumb(50,r,sd);
  h1=hs*h0;
  i1=arraytomat(int1[_m,1,ri,.])';
  i2=arraytomat(int2[_m,1,ri,.])';
  im=GaussRoughness(r)./h1+i0*n+i2*(n-1)-i1*2*n;
  m1[ri]=minc(im);
endfor;

for ri (1,rn,1);
  r=rs[ri];
  h0=PolyRuleThumb(50,r,s,sd);
  h1=hs*h0;
  i1=arraytomat(int1[_m,si+1,ri,.])';
  i2=arraytomat(int2[_m,si+1,ri,.])';
  im=PolyRoughness(r,s)./h1+i0*n+i2*(n-1)-i1*2*n;
  m2[ri]=minc(im);
endfor;

h0=TrapRuleThumb(50,sd);
h1=hs*h0;
i1=intd1[_m,.]';
i2=i1;
im=(1/pi)./h1+i0*n+i2*(n-1)-i1*2*n;
m3=minc(im);

i1=intt1[_m,.]';
i2=intt2[_m,.]';
im=(4/3/pi)./h1+i0*n+i2*(n-1)-i1*2*n;
m4=minc(im);

b=minc(m1|m2|m3|m4);
mg[_m,.]=(m1')/b;
mp[_m,.]=(m2')/b;
md[_m]=m3/b;
mt[_m]=m4/b;

endfor;

regret[ni,1]=maxc(mg[.,1]);

rg=maxc(mg);
regret[ni,2]=minc(rg);
regretr[ni,1]=minindc(rg);

rp=maxc(mp);
regret[ni,3]=minc(rp);
regretr[ni,2]=minindc(rp);

regret[ni,4]=maxc(md);
regret[ni,5]=maxc(mt);

endfor;

regret=ln(regret);


if minimax_r==0;

graphset;
pqgwin many;
xtics(0,1000,200,20);
ytics(0,.6,.1,10);
_pcolor=15;
fonts("complex simgrma");
_protate=1;
_plwidth=3;
xlabel("Sample Size");
ylabel("Regret");
title("Figure 5\LRegret Across Kernel Classes");
_pdate="";
let _plegctl= 1 5 0 .46;
let _pltype = 6 3 4 5 2;
_plegstr="\202F\201\000G]n[[*]\000K]n,8[[*]\000D\000\202l\201]2[";
xy(ns,regret);

elseif minimax_r==1;
"minimax r by sample size";
"(kernel order is 2r)";
"      n      Minimax r ";
ns~regretr[.,1];
"We have printed minimax r by sample size";
"(kernel order is 2r)";
endif;